package cn.estsh.i3plus.ext.mes.pcn.apiservice.schedulejob;


import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesWorkOrderService;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesProduceSn;
import cn.estsh.i3plus.pojo.mes.bean.MesProductionRecord;
import cn.estsh.i3plus.pojo.mes.repository.MesProductionRecordRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.base.schedule.BaseImppScheduleJob;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.init.ApplicationProperties;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.JobExecutionContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.List;

/**
 * @author castle,暂时没有客供品移库
 * @version 1.0
 * @date 2021/2/2 16:44
 **/
@Slf4j
@DisallowConcurrentExecution
@Component
@ApiOperation("非排序报工job")
public class MesReportNoSortJob extends BaseMesScheduleJob {

    @Autowired
    private IMesWorkOrderService workOrderService;

    @Autowired
    private MesProductionRecordRepository productionRecordRao;

    public MesReportNoSortJob() {
        super(MesReportNoSortJob.class, "非排序报工JOB");
    }
    @Override
    public void executeMesJob(JobExecutionContext context, ApplicationProperties applicationProperties) {
        JSONObject jsonObject= new JSONObject();

        jsonObject.putIfAbsent("organizeCode", "CK01");
        jsonObject.putIfAbsent("userName", "job");

        String organizeCode = jsonObject.getStr("organizeCode");
        String userName = jsonObject.getStr("userName");
        if (null == organizeCode){
            log.error("请添加需要报工的工厂代码!");
            return;
        }
        if (null == userName){
            userName = "JOB";
        }
        List<MesProduceSn> unReportMesProduceSn = workOrderService.getUnReportMesProduceSn(organizeCode);
        for (MesProduceSn mesProduceSn : unReportMesProduceSn) {
            try {
                workOrderService.doProductReport(Arrays.asList(mesProduceSn), organizeCode, userName);
                //1.获取生产加工记录production_record表中，report_status为待汇报状态的工单
                updateProductionRecord(organizeCode, userName, mesProduceSn);
            } catch (ImppBusiException e) {
                LOGGER.error("条码:{}报工失败", mesProduceSn.getProductSn(), e.getErrorDetail());
                updateProductionRecord(organizeCode, userName, mesProduceSn, e.getErrorDetail());
                continue;
            }
        }
    }

    private void updateProductionRecord(String organizeCode, String userName, MesProduceSn mesProduceSn) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);

        DdlPreparedPack.getStringEqualPack(mesProduceSn.getProductSn(), "productSn", ddlPackBean);
        productionRecordRao.updateByProperties(new String[]{"modifyUser", "modifyDatetime","reportStatus"},
                new Object[]{userName, TimeTool.getNowTime(true), MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_20.getValue() },ddlPackBean);
    }
    private void updateProductionRecord(String organizeCode, String userName, MesProduceSn mesProduceSn, String errorMsg) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);

        DdlPreparedPack.getStringEqualPack(mesProduceSn.getProductSn(), "productSn", ddlPackBean);
        productionRecordRao.updateByProperties(new String[]{"modifyUser", "modifyDatetime","reportStatus", "remark"},
                new Object[]{userName, TimeTool.getNowTime(true), MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_40.getValue(), errorMsg },ddlPackBean);
    }
}
